#ifndef S2HAT_MT_COMMON
#define S2HAT_MT_COMMON

// File: s2hat_c_common.h
// Created by Mikolaj Szydlarski (mikolaj.szydlarski@inria.fr) on 2011-06-09.
// 
// Copyright (C) 2012 Joel Falcou, Laura Grigori, Ovidiu Hupca, Radek Stompor, Mikolaj Szydlarski
// 
// This file is part of S2HAT_CUDA.
// 
// S2HAT_CUDA is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// S2HAT_CUDA is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with S2HAT_CUDA (see 'S2HAT_CUDA_LICENSE.txt' file). 
// If not, see <http://www.gnu.org/licenses/>.


#include <complex.h>
#include <fftw3.h>
#include <float.h>
#include <math.h>
#include <mpi.h>
#include <omp.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "s2hat_defs.h"
#include "s2hat.h"
#include "s2hat_utils.h"

#define max_(a,b) ( (a > b)?(a):(b) )
#define min_(a,b) ( (a < b)?(a):(b) )

typedef struct {
	s2hat_flt8* re;
	s2hat_flt8* im;
} ComplexStructPtr;            // complexV

typedef struct {
	s2hat_flt8 re;
	s2hat_flt8 im;
} ComplexStruct;              // complexFl

typedef struct {
	ComplexStruct pos;
	ComplexStruct neg;
} PHX_DoubleComplexStruct;

typedef struct {
	double complex pos;
	double complex neg;
} PHX_StdComplexStruct;

struct fftw_counters_c2r {
   
	int max_lenght;

	int fft_c2r_plan_set;
	int fft_call_count;
	int fft_full_count;
	
	// double complex* fft_vec_in;
   
	fftw_complex *fft_vec_in;
	fftw_plan fftw_c2r_plan;
};

struct fftw_counters_r2c {
	
	int max_lenght;
	
	int fft_r2c_plan_set;
	int fft_call_count;
	int fft_full_count;
	
	// double complex *fft_vec_out;
	
	fftw_complex *fft_vec_out;
	fftw_plan fftw_r2c_plan;
};

#define init_rescale_local init_rescale_local_double

#define HPX_MXL0 40    // minimum used, choose <=0 to do full loop
#define HPX_MXL1 1.35  // parameters of Ylm short-cut 

#define LOG2LG 100
#define LOG(X) log(X)
#define POWER(X,Y) pow(X,Y)
#define SQRT(X) sqrt(X) 

#define FL_LARGE	( pow(2.0,	LOG2LG  ) )
#define FL_SMALL	( pow(2.0,(-LOG2LG) ) )

#define absolute(X) fabs(X)

#define REAL_MAX DBL_MAX

#define RSMIN		-10
#define RSMAX		+10
#define RESCALE_TAB(X) ( rescale_tab[ X + smax ] )

#define ALN2_INV 1.4426950408889634073599246810						  /* 1/log(2) */
#define SQ4PI_INV 0.2820947917738781434740397257803862929220

void init_fft_r2c(struct fftw_counters_r2c *fft, int max_lenght);
void init_fft_c2r(struct fftw_counters_c2r *fft, int max_lenght);

void free_fft_r2c(struct fftw_counters_r2c *fft);
void free_fft_c2r(struct fftw_counters_c2r *fft);

int fix_mmap(int dir, int nmmax, int* indx, int nmvals, int* mvals, int myid, int numprocs, MPI_Comm comm);
void gen_recfac(int l_max, int m, ComplexStructPtr* recfac);

int s2hat_ring_synthesis(int nlmax, int nmmax, double complex* datain, int nph, s2hat_flt8* dataout, s2hat_flt8 kphi0, struct fftw_counters_c2r *fftwc);
int s2hat_ring_analysis(int nlmax, int nmmax, s2hat_flt8*  datain, int nph, double complex* dataout, s2hat_flt8 kphi0, struct fftw_counters_r2c *fftwc);

double __inline__ timesDiff(clock_t* stopm, clock_t* startm);
int __inline__ init_rescale_local_double(s2hat_flt8** rescale__tab);
int __inline__ l_min_ylm(int m, s2hat_flt8 sth);
void __inline__ mpi_compute_lam_mm(s2hat_flt8 mfac, int m, s2hat_flt8 sth, int smax, s2hat_flt8* rescale_tab, s2hat_flt8* lam_mm, s2hat_flt8* corfac, int* scalem);

#endif