#include "s2hat_cuda_tools.cuh"

// 
//  File: s2hat_cuda_tools.cu
//  Created by Ovidiu Hupca, Mikolaj Szydlarski, Giulio Fabbian on 2012-04-26.
// 
// Copyright (C) 2012 Joel Falcou, Laura Grigori, Ovidiu Hupca, Radek Stompor, Mikolaj Szydlarski
// 
// This file is part of S2HAT_CUDA.
// 
// S2HAT_CUDA is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// S2HAT_CUDA is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with S2HAT_CUDA (see 'S2HAT_CUDA_LICENSE.txt' file). 
// If not, see <http://www.gnu.org/licenses/>.

void Check_CUDA_Error(const char *message) {
	cudaError_t error = cudaGetLastError();
	if (error!=cudaSuccess) {
		fprintf(stderr,"ERROR: %s: %s\n", message, cudaGetErrorString(error) );
		exit(-1);
	}
}

void Check_CUFFT_Error(cufftResult res, const char *message, int iring) {
	switch (res) {
		case CUFFT_SUCCESS: return;	// all ok
		case CUFFT_ALLOC_FAILED:   { fprintf(stderr,"ERROR: %s: %s\n", message, "Allocation of GPU resources for the plan failed." ); return; }
		case CUFFT_INVALID_TYPE:   { fprintf(stderr,"ERROR: %s: %s\n", message, "The type parameter is not supported." ); return; }
		case CUFFT_INTERNAL_ERROR: { fprintf(stderr,"ERROR: %s: %s\n", message, "Internal driver error is detected." ); return; }
		case CUFFT_SETUP_FAILED:   { fprintf(stderr,"ERROR: %s: %s\n", message, "CUFFT library failed to initialize." ); return; }
		case CUFFT_INVALID_SIZE:   { fprintf(stderr,"ERROR: %s: %s\n", message, "The nx parameter is not a supported size." ); return; }
		case CUFFT_INVALID_PLAN:   { fprintf(stderr,"ERROR: %s: %s\n", message, "The plan parameter is not a valid handle." ); return; }
		case CUFFT_INVALID_VALUE:  { fprintf(stderr,"ERROR: %s: %s\n", message, "The idata, odata, and/or direction parameter is not valid." ); return; }
		case CUFFT_EXEC_FAILED:    { fprintf(stderr,"ERROR: %s: ring %d : %s\n", message, iring , "CUFFT failed to execute the transform on GPU." ); return; }
	 default: 		   { fprintf(stderr,"ERROR: %s: %s\n", message, "Unknown error" ); return; }
	}
}


// simple routine to assign mpi process to GPU device

int selectDevice(int process_id) {
	
	cudaThreadExit(); // Cleanup a device
	
	int devCount = 0;
	int dev = 0;
	
	cudaError_t res;
	
	res = cudaGetDeviceCount(&devCount);

	if (res != cudaSuccess) {
		perror("[e] --- Unable to list devices\n");
		return -1;	
	}
	
	dev = process_id % devCount;

	if (cudaSetDevice(dev) != cudaSuccess) {
		fprintf(stderr,"[e] --- Unable to select device %d/%d. %s\n", dev, devCount, (res == cudaErrorInvalidDevice) ? ("cudaErrorInvalidDevice") : ("ErrorSetOnActiveProcess") );
		return -1;	
	} else {
		return 0;  
	} 
}

